@@ -12,19 +12,21 @@ from functools import lru_cache, partial
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Callable, Match, Pattern, Sequence
 
-from griffe.dataclasses import Alias, Object
-from griffe.docstrings.dataclasses import (
+from griffe import (
+    Alias,
     DocstringSectionAttributes,
     DocstringSectionClasses,
     DocstringSectionFunctions,
     DocstringSectionModules,
+    Object,
 )
 from jinja2 import TemplateNotFound, pass_context, pass_environment
 from markupsafe import Markup
+from mkdocs_autorefs.references import AutorefsHookInterface
 from mkdocstrings.loggers import get_logger
 
 if TYPE_CHECKING:
-    from griffe.dataclasses import Attribute, Class, Function, Module
+    from griffe import Attribute, Class, Function, Module
     from jinja2 import Environment, Template
     from jinja2.runtime import Context
     from mkdocstrings.handlers.base import CollectorItem
@@ -294,7 +296,10 @@ def do_crossref(path: str, *, brief: bool = True) -> Markup:
     full_path = path
     if brief:
         path = full_path.split(".")[-1]
-    return Markup("<span data-autorefs-optional-hover={full_path}>{path}</span>").format(full_path=full_path, path=path)
+    return Markup("<autoref identifier={full_path} optional hover>{path}</autoref>").format(
+        full_path=full_path,
+        path=path,
+    )
 
 
 @lru_cache
@@ -326,7 +331,7 @@ def do_multi_crossref(text: str, *, code: bool = True) -> Markup:
         path = match.group()
         path_var = f"path{group_number}"
         variables[path_var] = path
-        return f"<span data-autorefs-optional-hover={{{path_var}}}>{{{path_var}}}</span>"
+        return f"<autoref identifier={{{path_var}}} optional hover>{{{path_var}}}</autoref>"
 
     text = re.sub(r"([\w.]+)", repl, text)
     if code:
@@ -364,11 +369,9 @@ def _keep_object(name: str, filters: Sequence[tuple[Pattern, bool]]) -> bool:
         if regex.search(name):
             keep = not exclude
     if keep is None:
-        if rules == {False}:
-            # only included stuff, no match = reject
-            return False
-        # only excluded stuff, or included and excluded stuff, no match = keep
-        return True
+        # When we only include stuff, no match = reject.
+        # When we only exclude stuff, or include and exclude stuff, no match = keep.
+        return rules != {False}
     return keep
 
 
@@ -569,3 +572,60 @@ def do_as_modules_section(
         A modules docstring section.
     """
     return DocstringSectionModules([])
+
+
+class AutorefsHook(AutorefsHookInterface):
+    """Autorefs hook.
+
+    With this hook, we're able to add context to autorefs (cross-references),
+    such as originating file path and line number, to improve error reporting.
+    """
+
+    def __init__(self, current_object: Object | Alias, config: dict[str, Any]) -> None:
+        """Initialize the hook.
+
+        Parameters:
+            current_object: The object being rendered.
+            config: The configuration dictionary.
+        """
+        self.current_object = current_object
+        self.config = config
+
+    def expand_identifier(self, identifier: str) -> str:
+        """Expand an identifier.
+
+        Parameters:
+            identifier: The identifier to expand.
+
+        Returns:
+            The expanded identifier.
+        """
+        return identifier
+
+    def get_context(self) -> AutorefsHookInterface.Context:
+        """Get the context for the current object.
+
+        Returns:
+            The context.
+        """
+        role = {
+            "attribute": "data" if self.current_object.parent and self.current_object.parent.is_module else "attr",
+            "class": "class",
+            "function": "meth" if self.current_object.parent and self.current_object.parent.is_class else "func",
+            "module": "mod",
+        }.get(self.current_object.kind.value.lower(), "obj")
+        origin = self.current_object.path
+        try:
+            filepath = self.current_object.docstring.parent.filepath  # type: ignore[union-attr]
+            lineno = self.current_object.docstring.lineno or 0  # type: ignore[union-attr]
+        except AttributeError:
+            filepath = self.current_object.filepath
+            lineno = 0
+
+        return AutorefsHookInterface.Context(
+            domain="py",
+            role=role,
+            origin=origin,
+            filepath=str(filepath),
+            lineno=lineno,
+        )
